/**
 * @file
 * Drupal File plugin.
 *
 * @ignore
 */

(function ($, Drupal, drupalSettings, CKEDITOR) {

  "use strict";
  /**
   *
   * Fuction to parse attributes CKEditor file.
   */

  function parseAttributes(element) {
    var parsedAttributes = {};

    var domElement = element.$;
    var attribute = null;
    var attributeName;
    for (var attrIndex = 0; attrIndex < domElement.attributes.length; attrIndex++) {
      attribute = domElement.attributes.item(attrIndex);
      attributeName = attribute.nodeName.toLowerCase();
      // Don't consider data-cke-saved- attributes; they're just there to work
      // around browser quirks.
      if (attributeName.substring(0, 15) === 'data-cke-saved-') {
        continue;
      }
      // Store the value for this attribute, unless there's a data-cke-saved-
      // alternative for it, which will contain the quirk-free, original value.
      parsedAttributes[attributeName] = element.data('cke-saved-' + attributeName) || attribute.nodeValue;
    }
    return parsedAttributes;
  }

  /**
   *
   * Function to construct css style.
   */

  function getStyle(style, valor, flag = false) {
    if (valor.trim() != '') {
      if (flag) {
        return style + ':' + "'" + valor + "';";
      }
      return style + ':' + valor + ";";
    }
    return '';
  }

  CKEDITOR.plugins.add('bgimage', {
    init: function (editor) {
      // Add the commands for file and unfile.
      editor.addCommand('bgimage', {
        allowedContent: {
          a: {
            attributes: {
              '!href': true,
              '!href': true,
              '!data-entity-type': true,
              '!data-entity-uuid': true
            },
            classes: {}
          }
        },
        requiredContent: new CKEDITOR.style(
          {
            element: 'a',
            attributes: {
              'href': '',
              'data-entity-type': '',
              'data-entity-uuid': ''
              }
          }
        ),
        modes: {wysiwyg: 1},
        canUndo: true,
        exec: function (editor) {
          var drupalImageUtils = CKEDITOR.plugins.drupalimage;
          var focusedImageWidget = drupalImageUtils && drupalImageUtils.getFocusedWidget(editor);
          var fileElement = getSelectedFile(editor);

          // Set existing values based on selected element.
          var existingValues = {};
          if (fileElement && fileElement.$) {
            existingValues = parseAttributes(fileElement);
          }
          // Or, if an image widget is focused, we're editing a link wrapping
          // an image widget.
          else if (focusedImageWidget && focusedImageWidget.data.link) {
            existingValues = CKEDITOR.tools.clone(focusedImageWidget.data.link);
          }

          // Prepare a save callback to be used upon saving the dialog.
          var saveCallback = function (returnValues) {
            editor.fire('saveSnapshot');

            /* config_image */
            let image = returnValues.attributes.image;
            var background_color = returnValues.background_color;
            let width = '100%';
            let height = '100%';
            let position = returnValues.background_aling;
            if(returnValues.width != ''){
              width = returnValues.width+'px';
            }
            if(returnValues.height != ''){
              height = returnValues.height+'px';
            }
            let content = editor.document.getBody().getHtml();
            let matches = content.match(/<div style="(.*)">((.|\n)*?)<\/div>/)
            // styled div already exists
            if (matches) {
              content = matches[2];
            }

            let div = '<div style="';
            div += 'background-image: url(' + image + ');';
            div += 'background-color: '+ background_color +';';
            div += 'background-repeat: no-repeat;';
            div += 'background-position: '+ position +';';
            div += 'padding:25px;'
            div += 'background-size:' + width + ' ' + height + ';';
            div += '">';
            div += content;
            div += '</div>'
            editor.setData(div);

            // Save snapshot for undo support.
            editor.fire('saveSnapshot');
          };
          // Drupal.t() will not work inside CKEditor plugins because CKEditor
          // loads the JavaScript file instead of Drupal. Pull translated
          // strings from the plugin settings that are translated server-side.
          var dialogSettings = {
            title: fileElement ? editor.config.bgimage_dialogTitleEdit : editor.config.bgimage_dialogTitleAdd,
            dialogClass: 'editor-file-dialog'
          };

          //console.log(editor.config.drupal.format);
          // Open the dialog for the edit form.
          Drupal.ckeditor.openDialog(editor, Drupal.url('ckeditor_bgimage/dialog/file/' + editor.config.drupal.format), existingValues, saveCallback, dialogSettings);
        }
      });

      // Add buttons for file upload.
      if (editor.ui.addButton) {
        editor.ui.addButton('bgimage', {
          label: 'bgimage',
          command: 'bgimage',
          toolbar:'insert',
          icon: this.path + 'icons/background.png'
        });
      }


      editor.on('doubleclick', function (evt) {
        var element = getSelectedFile(editor) || evt.data.element;

        if (!element.isReadOnly()) {
          if (element.is('a') && element.getAttribute('data-entity-uuid')) {
            editor.getSelection().selectElement(element);
            editor.getCommand('bgimage').exec();
          }
        }
      });

      // If the "menu" plugin is loaded, register the menu items.
      if (editor.addMenuItems) {
        editor.addMenuItems({
          file: {
            label: Drupal.t('Edit File'),
            command: 'bgimage',
            group: 'link',
            order: 1
          }
        });
      }

      // If the "contextmenu" plugin is loaded, register the listeners.
      if (editor.contextMenu) {
        editor.contextMenu.addListener(function (element, selection) {
          if (!element || element.isReadOnly()) {
            return null;
          }
          var anchor = getSelectedFile(editor);
          if (!anchor) {
            return null;
          }

          var menu = {};
          if (anchor.getAttribute('href') && anchor.getChildCount()) {
            menu = {file: CKEDITOR.TRISTATE_OFF};
          }
          return menu;
        });
      }
    }
  });

  /**
   * Get the surrounding file element of current selection.
   *
   * The following selection will all return the file element.
   *
   * @example
   *  <a href="#">li^nk</a>
   *  <a href="#">[file]</a>
   *  text[<a href="#">file]</a>
   *  <a href="#">li[nk</a>]
   *  [<b><a href="#">li]nk</a></b>]
   *  [<a href="#"><b>li]nk</b></a>
   *
   * @param {CKEDITOR.editor} editor
   *   The CKEditor editor object
   *
   * @return {?HTMLElement}
   *   The selected file element, or null.
   *
   */
  function getSelectedFile(editor) {
    var selection = editor.getSelection();
    var selectedElement = selection.getSelectedElement();
    if (selectedElement && selectedElement.is('a')) {
      return selectedElement;
    }

    var range = selection.getRanges(true)[0];

    if (range) {
      range.shrink(CKEDITOR.SHRINK_TEXT);
      return editor.elementPath(range.getCommonAncestor()).contains('a', 1);
    }
    return null;
  }

})(jQuery, Drupal, drupalSettings, CKEDITOR);